Package: merTools
Title: Tools for Analyzing Mixed Effect Regression Models
Version: 1.0.0
Authors@R: c(
    person(c("Jared", "E."), "Knowles", email = "jared@civilytics.com", role = c("aut", "cre")),
    person("Carl", "Frederick", email="carlbfrederick@gmail.com", role = c("aut")),
    person("Alex", "Whitworth", email="whitworth.alex@gmail.com", role = c("ctb")))
Description: Provides methods for extracting results from mixed-effect model
    objects fit with the 'lme4' package. Allows construction of prediction intervals
    efficiently from large scale linear and generalized linear mixed-effects models.
    This method draws from the simulation framework used in the Gelman and Hill (2007) textbook:
    Data Analysis Using Regression and Multilevel/Hierarchical Models.
Depends: R (>= 3.0.2), lme4 (>= 1.1-11), methods
Suggests: testthat, knitr, rmarkdown, shiny, nlme, future.apply,
        rstanarm, Amelia, DT
Imports: arm, dplyr, mvtnorm, reformulas, foreach, abind, ggplot2,
        blme, broom.mixed, Matrix
License: GPL (>= 2)
LazyData: true
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Encoding: UTF-8
URL: https://jknowles.github.io/merTools/,
        https://github.com/jknowles/merTools
BugReports: https://github.com/jknowles/merTools/issues
Config/testthat/edition: 3
Config/testthat/parallel: false
NeedsCompilation: no
Packaged: 2026-05-30 20:25:08 UTC; jared
Author: Jared E. Knowles [aut, cre],
  Carl Frederick [aut],
  Alex Whitworth [ctb]
Maintainer: Jared E. Knowles <jared@civilytics.com>
Repository: CRAN
Date/Publication: 2026-05-31 05:11:49 UTC
Built: R 4.6.0; ; 2026-05-31 08:25:06 UTC; unix
