# Generated by roxygen2: do not edit by hand

S3method(VarCorr,merModList)
S3method(draw,merMod)
S3method(fastdisp,merMod)
S3method(fastdisp,merModList)
S3method(fixef,merModList)
S3method(print,merModList)
S3method(print,summary.merModList)
S3method(ranef,merModList)
S3method(summary,merModList)
export(FEsim)
export(ICC)
export(REcorrExtract)
export(REextract)
export(REimpact)
export(REmargins)
export(REquantile)
export(REsdExtract)
export(REsim)
export(RMSE.merMod)
export(averageObs)
export(bglmerModList)
export(blmerModList)
export(draw)
export(expectedRank)
export(fastdisp)
export(findFormFuns)
export(glmerModList)
export(lmerModList)
export(modelFixedEff)
export(modelInfo)
export(modelRandEffStats)
export(plotFEsim)
export(plotREimpact)
export(plotREsim)
export(predictInterval)
export(randomObs)
export(shinyMer)
export(subBoot)
export(superFactor)
export(thetaExtract)
export(wiggle)
import(dplyr)
import(ggplot2)
import(lme4)
importFrom(abind,abind)
importFrom(arm,fround)
importFrom(arm,pfround)
importFrom(arm,sigma.hat)
importFrom(arm,sim)
importFrom(blme,bglmer)
importFrom(blme,blmer)
importFrom(broom.mixed,tidy)
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(methods,as)
importFrom(methods,is)
importFrom(mvtnorm,rmvnorm)
importFrom(stats,AIC)
importFrom(stats,as.formula)
importFrom(stats,delete.response)
importFrom(stats,df)
importFrom(stats,formula)
importFrom(stats,getCall)
importFrom(stats,logLik)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,na.pass)
importFrom(stats,pnorm)
importFrom(stats,pt)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,reformulate)
importFrom(stats,reshape)
importFrom(stats,residuals)
importFrom(stats,rgamma)
importFrom(stats,rnorm)
importFrom(stats,rpois)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,terms)
importFrom(stats,vcov)
importFrom(stats,weighted.mean)
importFrom(utils,packageVersion)
