% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merList.R
\name{merModList-class}
\alias{merModList-class}
\alias{merModList}
\title{merModList S3 Class}
\description{
A list of fitted mixed-effects models from lme4
}
