% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predictInterval_helpers.R
\name{combine_components}
\alias{combine_components}
\title{Combine fixed, random and residual components into a prediction array}
\usage{
combine_components(
  fixed_mat,
  random_list,
  sigma_vec,
  include.resid.var = TRUE,
  which = c("full", "fixed", "random", "all"),
  family = NULL,
  link = NULL,
  weights = NULL,
  use.probability = FALSE
)
}
\arguments{
\item{fixed_mat}{Matrix of dimensions `nrow(newdata)` × `n.sims`.}

\item{random_list}{List returned by `simulate_random_effects()`.}

\item{sigma_vec}{Residual standard deviations (output of `simulate_residual_variance`).}

\item{include.resid.var}{Logical, whether to add residual variance.}

\item{which}{Character: "full", "fixed", "random" or "all".}
}
\value{
Array `nrow(newdata)` × `n.sims` (or list when `which="all"`).
}
\description{
Combine fixed, random and residual components into a prediction array
}
\keyword{internal}
