% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{levelfun}
\alias{levelfun}
\title{Parse merMod levels}
\usage{
levelfun(x, nl.n, allow.new.levels = FALSE)
}
\description{
Parse merMod levels
}
\keyword{internal}
