% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merData.R
\name{collapseFrame}
\alias{collapseFrame}
\title{Collapse a dataframe to a single average row}
\usage{
collapseFrame(data)
}
\arguments{
\item{data}{a data.frame}
}
\value{
a data frame with a single row
}
\description{
Take an entire dataframe and summarize it in one row by using the
mean and mode.
}
\details{
Each character and factor variable in the data.frame is assigned to the
modal category and each numeric variable is collapsed to the mean. Currently if
mode is a tie, returns a "."
}
