% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{buildModelMatrix}
\alias{buildModelMatrix}
\title{Build model matrix}
\source{
Taken from predict.merMod in lme4
}
\usage{
buildModelMatrix(model, newdata, which = "full")
}
\arguments{
\item{model}{a merMod object from lme4}

\item{newdata}{a data frame to construct the matrix from}

\item{which}{a character which matrix to return,default is full matrix with fixed and
random terms, other options are "fixed" and "random"}
}
\description{
a function to create a model matrix with all predictor terms in
both the group level and fixed effect level
}
\keyword{internal}
