% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parallel.R
\name{setup_parallel}
\alias{setup_parallel}
\title{Set up parallel environment}
\usage{
setup_parallel()
}
\value{
Nothing
}
\description{
Set up parallel environment
}
